@@ -1,49 +1,95 @@
-from geraldo.utils import get_attr_value, calculate_size
+import random, shelve, os
+
+from geraldo.utils import get_attr_value, calculate_size, memoize
 from geraldo.widgets import Widget, Label, SystemField
 from geraldo.graphics import Graphic, RoundRect, Rect, Line, Circle, Arc,\
         Ellipse, Image
+from geraldo.barcodes import BarCode
+from geraldo.base import GeraldoObject, ManyElements
+from geraldo.cache import CACHE_BY_QUERYSET, CACHE_BY_RENDER, CACHE_DISABLED,\
+        make_hash_key, get_cache_backend
+from geraldo.charts import BaseChart
+from geraldo.exceptions import AbortEvent
 
-class ReportPage(object):
+class ReportPage(GeraldoObject):
     rect = None
-    elements = None
+    _elements = None
     width = None
+    randomic_number = None
 
     def __init__(self):
-        self.elements = []
+        self._elements = []
 
-class ReportGenerator(object):
+        self.randomic_number = str(random.randint(1, 999999)).zfill(6)
+
+    def get_children(self):
+        return self._elements
+
+    def add_element(self, el):
+        """In future this method can be used to store pages on disk and reduce
+        memory consuming"""
+        self._elements.append(el)
+
+    @property
+    def elements(self):
+        """In future this method can be used to restore pages from disk and reduce
+        memory consuming"""
+        for el in self._elements:
+            yield el
+
+    @memoize
+    def repr_for_cache_hash_key(self):
+        return '/'.join([el.repr_for_cache_hash_key() for el in self.elements
+            if hasattr(el, 'repr_for_cache_hash_key')])
+
+class ReportGenerator(GeraldoObject):
     """A report generator is used to generate a report to a specific format."""
 
+    cache_enabled = None
+    first_page_number = 1
+    variables = None
+    return_pages = False
+
     _is_first_page = True
     _is_latest_page = True
     _current_top_position = 0
     _current_left_position = 0
-    _current_page_number = 0
+    _current_page_number = 1 # This variable is just used for generating, so, keep in mind it
+                             # has't the current number while rendering
     _current_object = None
     _current_queryset = None
     _generation_datetime = None
+    _highest_height = 0
 
     # Groupping
     _groups_values = None
-    _groups_previous_values = None
+    _groups_working_values = None
     _groups_changed = None
     _groups_stack = None
 
-    # The rendered report have pages, each page is a ReportPage instance
+    # The rendered report has pages, each page is a ReportPage instance
     _rendered_pages = None
     _page_rect = None
 
-    def __init__(self, report):
+    def __init__(self, report, first_page_number=1, variables=None, return_pages=False, pages=None, **kwargs):
         """This method should be overrided to receive others arguments"""
         self.report = report
 
         # Initializes some attributes
-        self._rendered_pages = []
+        self._rendered_pages = pages or []
+        self._current_page_number = len(self._rendered_pages)
         self._groups_values = {}
-        self._groups_previous_values = {}
+        self._groups_working_values = {}
         self._groups_changed = {}
         self._groups_stack = []
 
+        self.first_page_number = first_page_number
+        self.variables = variables or self.variables or {}
+        self.return_pages = return_pages
+
+    def get_children(self):
+        return self._rendered_pages
+
     def execute(self):
         """This method must be overrided to execute the report generation."""
 
@@ -61,7 +107,11 @@ class ReportGenerator(object):
                     width=rect_dict['right'] - rect_dict['left'],
                     height=rect_dict['height'],
                     )
-            self._rendered_pages[-1].elements.append(graphic)
+            # If border is a number, it is recognized as the stroke width
+            if isinstance(b_all, (int, float)):
+                graphic.stroke_width = b_all
+
+            self._rendered_pages[-1].add_element(graphic)
 
         b_left = borders_dict.get('left', None)
         if b_left:
@@ -70,7 +120,11 @@ class ReportGenerator(object):
                     left=rect_dict['left'], top=rect_dict['top'],
                     right=rect_dict['left'], bottom=rect_dict['bottom']
                     )
-            self._rendered_pages[-1].elements.append(graphic)
+            # If border is a number, it is recognized as the stroke width
+            if isinstance(b_left, (int, float)):
+                graphic.stroke_width = b_left
+
+            self._rendered_pages[-1].add_element(graphic)
 
         b_top = borders_dict.get('top', None)
         if b_top:
@@ -79,7 +133,11 @@ class ReportGenerator(object):
                     left=rect_dict['left'], top=rect_dict['top'],
                     right=rect_dict['right'], bottom=rect_dict['top']
                     )
-            self._rendered_pages[-1].elements.append(graphic)
+            # If border is a number, it is recognized as the stroke width
+            if isinstance(b_top, (int, float)):
+                graphic.stroke_width = b_top
+
+            self._rendered_pages[-1].add_element(graphic)
 
         b_right = borders_dict.get('right', None)
         if b_right:
@@ -88,7 +146,11 @@ class ReportGenerator(object):
                     left=rect_dict['right'], top=rect_dict['top'],
                     right=rect_dict['right'], bottom=rect_dict['bottom']
                     )
-            self._rendered_pages[-1].elements.append(graphic)
+            # If border is a number, it is recognized as the stroke width
+            if isinstance(b_right, (int, float)):
+                graphic.stroke_width = b_right
+
+            self._rendered_pages[-1].add_element(graphic)
 
         b_bottom = borders_dict.get('bottom', None)
         if b_bottom:
@@ -97,7 +159,11 @@ class ReportGenerator(object):
                     left=rect_dict['left'], top=rect_dict['bottom'],
                     right=rect_dict['right'], bottom=rect_dict['bottom']
                     )
-            self._rendered_pages[-1].elements.append(graphic)
+            # If border is a number, it is recognized as the stroke width
+            if isinstance(b_bottom, (int, float)):
+                graphic.stroke_width = b_bottom
+
+            self._rendered_pages[-1].add_element(graphic)
 
     def make_band_rect(self, band, top_position, left_position):
         """Returns the right band rect on the PDF canvas"""
@@ -109,12 +175,162 @@ class ReportGenerator(object):
                 'height': self.calculate_size(band.height),
                 }
         return band_rect
+ 
+    def make_widget_rect(self, widget, band_rect):
+        """Returns the right widget rect on the PDF canvas"""
+        widget_rect = {
+                'left': band_rect['left'] + calculate_size(widget.left),
+                'top': band_rect['top'] - calculate_size(widget.top),
+                'right': band_rect['left'] + calculate_size(widget.left) + calculate_size(widget.width),
+                'bottom': band_rect['top'] - calculate_size(widget.top) + calculate_size(widget.height),
+                'height': calculate_size(widget.height),
+                'width': calculate_size(widget.width),
+                }
+        return widget_rect
+
+    def render_element(self, element, current_object, band, band_rect, temp_top,
+            top_position):
+        # Doesn't render not visible element
+        if not element.visible:
+            return
+
+        # Widget element
+        if isinstance(element, Widget):
+            widget = element.clone()
+
+            # Set widget colors
+            widget.font_color = self.report.default_font_color
+
+            # Set widget basic attributes
+            widget.instance = current_object
+            widget.generator = self
+            widget.report = self.report # This should be done by a metaclass in Band domain TODO
+            widget.band = band # This should be done by a metaclass in Band domain TODO
+            widget.page = self._rendered_pages[-1]
+
+            # Border rect
+            widget_rect = self.make_widget_rect(widget, band_rect)
+
+            if isinstance(widget, SystemField):
+                widget.left = band_rect['left'] + self.calculate_size(widget.left)
+                widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top))
+
+                temp_height = self.calculate_size(element.top) + self.calculate_size(widget.height)
+            elif isinstance(widget, Label):
+                para = self.make_paragraph(widget.text, self.make_paragraph_style(band, widget.style))
+
+                if widget.truncate_overflow:
+                    self.keep_in_frame(
+                            widget,
+                            self.calculate_size(widget.width),
+                            self.calculate_size(widget.height),
+                            [para],
+                            mode='truncate',
+                            )
+
+                    widget.left = band_rect['left'] + self.calculate_size(widget.left)
+                    widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top), self.calculate_size(widget.height))
+                else:
+                    self.wrap_paragraph_on(para, self.calculate_size(widget.width), self.calculate_size(widget.height))
+                    widget.left = band_rect['left'] + self.calculate_size(widget.left)
+                    widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top), self.calculate_size(para.height))
+
+                temp_height = self.calculate_size(element.top) + self.calculate_size(para.height)
+            else:
+                temp_height = self.calculate_size(element.top) + self.calculate_size(widget.height)
+
+            # Sets element height as the highest
+            if temp_height > self._highest_height:
+                self._highest_height = temp_height
+
+            self._rendered_pages[-1].add_element(widget)
+
+            # Borders
+            self.render_border(widget.borders or {}, widget_rect)
+
+        # Graphic element
+        elif isinstance(element, Graphic):
+            graphic = element.clone()
+
+            # Set widget basic attributes
+            graphic.instance = current_object
+            graphic.generator = self
+            graphic.report = self.report # This should be done by a metaclass in Band domain TODO
+            graphic.band = band # This should be done by a metaclass in Band domain TODO
+            graphic.page = self._rendered_pages[-1]
+
+            # Set graphic colors
+            graphic.fill_color = graphic.fill_color or self.report.default_fill_color
+            graphic.stroke_color = graphic.stroke_color or self.report.default_stroke_color
+
+            if isinstance(graphic, RoundRect):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
+            elif isinstance(graphic, Rect):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
+            elif isinstance(graphic, Line):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top)
+                graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
+                graphic.bottom = top_position - self.calculate_size(graphic.bottom)
+            elif isinstance(graphic, Circle):
+                graphic.left_center = band_rect['left'] + self.calculate_size(graphic.left_center)
+                graphic.top_center = top_position - self.calculate_size(graphic.top_center)
+            elif isinstance(graphic, Arc):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top)
+                graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
+                graphic.bottom = top_position - self.calculate_size(graphic.bottom)
+            elif isinstance(graphic, Ellipse):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top)
+                graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
+                graphic.bottom = top_position - self.calculate_size(graphic.bottom)
+            elif isinstance(graphic, Image):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
+            elif isinstance(graphic, BarCode):
+                barcode = graphic.render()
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
+                self.wrap_barcode_on(barcode, graphic.width, graphic.height)
+            elif isinstance(element, BaseChart):
+                graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
+                graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
+
+            # Sets element height as the highest
+            temp_height = self.calculate_size(element.top) + self.calculate_size(graphic.height)
+            if temp_height > self._highest_height:
+                self._highest_height = temp_height
+
+            self._rendered_pages[-1].add_element(graphic)
+
+        # Many elements
+        elif isinstance(element, ManyElements):
+            # Set widget basic attributes
+            element.instance = current_object
+            element.generator = self
+            element.report = self.report # This should be done by a metaclass in Band domain TODO
+            element.band = band # This should be done by a metaclass in Band domain TODO
+            element.page = self._rendered_pages[-1]
+
+            # Get the elements and render them
+            for el in element.get_elements():
+                self.render_element(el, current_object, band, band_rect, temp_top, top_position)
+
 
     def render_band(self, band, top_position=None, left_position=None,
             update_top=True, current_object=None):
         """Generate a band having the current top position or informed as its
         top coordinate"""
 
+        # Calls the before_print event
+        try:
+            band.do_before_print(generator=self)
+        except AbortEvent:
+            return False
+
         # Sets the current object
         current_object = current_object or self._current_object
 
@@ -147,116 +363,17 @@ class ReportGenerator(object):
         self.render_border(band.borders, band_rect)
 
         # Variable that stores the highest height at all elements
-        highest_height = 0
+        self._highest_height = 0
 
         # Loop at band widgets
         for element in band.elements:
-            # Doesn't render not visible element
-            if not element.visible:
-                continue
-
-            # Widget element
-            if isinstance(element, Widget):
-                widget = element.clone()
-
-                # Set widget colors
-                widget.font_color = self.report.default_font_color
-
-                # Set widget basic attributes
-                widget.instance = current_object
-                widget.generator = self
-                widget.report = self.report # This should be done by a metaclass in Band domain TODO
-                widget.band = band # This should be done by a metaclass in Band domain TODO
-                widget.page = self._rendered_pages[-1]
-
-                if isinstance(widget, SystemField):
-                    widget.left = band_rect['left'] + self.calculate_size(widget.left)
-                    widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top))
-
-                    temp_height = self.calculate_size(element.top) + self.calculate_size(widget.height)
-                elif isinstance(widget, Label):
-                    widget.para = self.make_paragraph(widget.text, self.make_paragraph_style(band, widget.style))
-
-                    if widget.truncate_overflow:
-                        self.keep_in_frame(
-                                widget,
-                                self.calculate_size(widget.width),
-                                self.calculate_size(widget.height),
-                                [widget.para],
-                                mode='truncate',
-                                )
-
-                        widget.left = band_rect['left'] + self.calculate_size(widget.left)
-                        widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top), self.calculate_size(widget.height))
-                    else:
-                        self.wrap_paragraph_on(widget.para, self.calculate_size(widget.width), self.calculate_size(widget.height))
-                        widget.left = band_rect['left'] + self.calculate_size(widget.left)
-                        widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top), self.calculate_size(widget.para.height))
-
-                    temp_height = self.calculate_size(element.top) + self.calculate_size(widget.para.height)
-                else:
-                    temp_height = self.calculate_size(element.top) + self.calculate_size(widget.height)
-
-                # Sets element height as the highest
-                if temp_height > highest_height:
-                    highest_height = temp_height
-
-                self._rendered_pages[-1].elements.append(widget)
-
-            # Graphic element
-            elif isinstance(element, Graphic):
-                graphic = element.clone()
-
-                # Set widget basic attributes
-                graphic.instance = current_object
-                graphic.generator = self
-                graphic.report = self.report # This should be done by a metaclass in Band domain TODO
-                graphic.band = band # This should be done by a metaclass in Band domain TODO
-                graphic.page = self._rendered_pages[-1]
-
-                # Set graphic colors
-                graphic.fill_color = graphic.fill_color or self.report.default_fill_color
-                graphic.stroke_color = graphic.stroke_color or self.report.default_stroke_color
-
-                if isinstance(graphic, RoundRect):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
-                elif isinstance(graphic, Rect):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
-                elif isinstance(graphic, Line):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top)
-                    graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
-                    graphic.bottom = top_position - self.calculate_size(graphic.bottom)
-                elif isinstance(graphic, Circle):
-                    graphic.left_center = band_rect['left'] + self.calculate_size(graphic.left_center)
-                    graphic.top_center = top_position - self.calculate_size(graphic.top_center)
-                elif isinstance(graphic, Arc):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top)
-                    graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
-                    graphic.bottom = top_position - self.calculate_size(graphic.bottom)
-                elif isinstance(graphic, Ellipse):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top)
-                    graphic.right = band_rect['left'] + self.calculate_size(graphic.right)
-                    graphic.bottom = top_position - self.calculate_size(graphic.bottom)
-                elif isinstance(graphic, Image):
-                    graphic.left = band_rect['left'] + self.calculate_size(graphic.left)
-                    graphic.top = top_position - self.calculate_size(graphic.top) - self.calculate_size(graphic.height)
-
-                # Sets element height as the highest
-                temp_height = self.calculate_size(element.top) + self.calculate_size(graphic.height)
-                if temp_height > highest_height:
-                    highest_height = temp_height
-
-                self._rendered_pages[-1].elements.append(graphic)
+            self.render_element(element, current_object, band, band_rect, temp_top,
+                    top_position)
 
         # Updates top position
         if update_top:
             if band.auto_expand_height:
-                band_height = highest_height
+                band_height = self._highest_height
             else:
                 band_height = self.calculate_size(band.height)
 
@@ -281,12 +398,20 @@ class ReportGenerator(object):
 
             self.render_band(child_band)
 
+        # Calls the before_print event
+        band.do_after_print(generator=self)
+
+        return True
+
     def force_blank_page_by_height(self, height):
         """Check if the height is in client available report height and
         makes a new page if necessary"""
         if self.get_available_height() < height:
             self.force_new_page()
 
+    def append_new_page(self):
+        self._rendered_pages.append(ReportPage())
+
     def force_new_page(self, insert_new_page=True):
         """Starts a new blank page"""
         # Ends the current page
@@ -294,7 +419,7 @@ class ReportGenerator(object):
 
         # Creates the new page
         if insert_new_page:
-            self._rendered_pages.append(ReportPage())
+            self.append_new_page()
 
         # Starts a new one
         self.start_new_page()
@@ -382,12 +507,13 @@ class ReportGenerator(object):
  
     def render_bands(self):
         """Loops into the objects list to create the report pages until the end"""
+ 
         # Preparing local auxiliar variables
-        self._current_page_number = 0
+        self._current_page_number = self.report.first_page_number
         self._current_object_index = 0
         objects = self.report.get_objects_list()
 
-        # just an alias to make it easier
+        # just an alias to make it shorter
         d_band = self.report.band_detail
 
         # Empty report
@@ -403,7 +529,7 @@ class ReportGenerator(object):
             first_object_on_page = True
 
             # Generate the report begin band
-            if self._current_page_number == 0:
+            if self._is_first_page:
                 self.render_begin()
 
             # Does generate objects if there is no details band
@@ -419,16 +545,27 @@ class ReportGenerator(object):
                 self.calc_changed_groups(first_object_on_page)
 
                 if not first_object_on_page:
+                    # The current_object of the groups' footers is the previous 
+                    # object, so we have access, in groups' footers, to the last
+                    # object before the group breaking
+                    self._current_object = objects[self._current_object_index-1]
                     self.render_groups_footers()
+                    self._current_object = objects[self._current_object_index]
 
                 self.render_groups_headers()
 
                 # Generate this band only if it is visible
+                # - "done True" means band was rendered ok
+                # - "done False" means band rendering was aborted
+                # - "done None" means band didn't render, but wasn't aborted
                 if d_band.visible:
-                    self.render_band(d_band)
+                    done = self.render_band(d_band)
+                else:
+                    done = None
 
                 # Renders subreports
-                self.render_subreports()
+                if done != False:
+                    self.render_subreports()
 
                 # Next object
                 self._current_object_index += 1
@@ -436,18 +573,19 @@ class ReportGenerator(object):
 
                 # Break this if this page doesn't suppport nothing more...
                 # ... if there is no more available height
-                if self.get_available_height() < self.calculate_size(d_band.height):
-                    # right margin is not considered to calculate the necessary space
-                    d_width = self.calculate_size(d_band.width) + self.calculate_size(getattr(d_band, 'margin_left', 0))
+                if done != False:
+                    if self.get_available_height() < self.calculate_size(d_band.height):
+                        # right margin is not considered to calculate the necessary space
+                        d_width = self.calculate_size(d_band.width) + self.calculate_size(getattr(d_band, 'margin_left', 0))
 
-                    # ... and this is not an inline displayed detail band or there is no width available
-                    if not getattr(d_band, 'display_inline', False) or self.get_available_width() < d_width:
+                        # ... and this is not an inline displayed detail band or there is no width available
+                        if not getattr(d_band, 'display_inline', False) or self.get_available_width() < d_width:
+                            break
+
+                    # ... or this band forces a new page and this is not the last object in objects list
+                    elif d_band.force_new_page and self._current_object_index < len(objects):
                         break
 
-                # ... or this band forces a new page and this is not the last object in objects list
-                elif d_band.force_new_page and self._current_object_index < len(objects):
-                    break
-
             # Sets this is the latest page or not
             self._is_latest_page = self._current_object_index >= len(objects)
 
@@ -468,7 +606,13 @@ class ReportGenerator(object):
 
     def start_new_page(self, with_header=True):
         """Do everything necessary to be done to start a new page"""
-        self._rendered_pages.append(ReportPage())
+        self.append_new_page()
+
+        self.report.do_on_new_page(
+                page=self._rendered_pages[-1],
+                page_number=len(self._rendered_pages) + self.first_page_number - 1,
+                generator=self,
+                )
 
         if with_header:
             self.render_page_header()
@@ -556,6 +700,10 @@ class ReportGenerator(object):
         """Wraps the paragraph on the height/width informed"""
         raise Exception('Not implemented')
 
+    def wrap_barcode_on(self, barcode, width, height):
+        """Wraps the barcode on the height/width informed"""
+        raise Exception('Not implemented')
+
     # Stylizing
 
     def set_fill_color(self, color):
@@ -578,7 +726,7 @@ class ReportGenerator(object):
         changed = force_no_changed
 
         # Stores the previous group values
-        self._groups_previous_values = self._groups_values.copy()
+        self._groups_working_values = self._groups_values.copy()
 
         # Loops on groups until find the first changed, then all under it are considered
         # changed also
@@ -601,6 +749,9 @@ class ReportGenerator(object):
         """Renders the report headers using 'changed' definition calculated by
         'calc_changed_groups'"""
 
+        # Update working values for groups
+        self._groups_working_values = self._groups_values
+
         # Loops on groups to render changed ones
         for group in self.report.groups:
             if self._groups_changed.get(group, None) and\
@@ -613,21 +764,18 @@ class ReportGenerator(object):
         """Renders the report footers using previous 'changed' definition calculated by
         'calc_changed_groups'"""
 
-        reversed_groups = [group for group in self.report.groups]
-        reversed_groups.reverse()
-
         # Loops on groups to render changed ones
-        for group in reversed_groups:
+        for group in reversed(self.report.groups):
             if force or ( self._groups_changed.get(group, None) and\
                           self._groups_stack and\
                           self._groups_stack[-1] == group ):
-                #if not force and (not self._groups_stack or self._groups_stack[-1] != group):
-                #    continue
-                
                 if group.band_footer and group.band_footer.visible:
                     self.force_blank_page_by_height(self.calculate_size(group.band_footer.height))
                     self.render_band(group.band_footer)
 
+                if self._groups_stack:
+                    self._groups_working_values.pop(self._groups_stack[-1])
+
                 self._groups_stack.pop()
 
     def get_current_queryset(self):
@@ -649,17 +797,16 @@ class ReportGenerator(object):
         """Returns objects filtered in the current group or all if there is no
         group"""
 
-        filter = dict([(group.attribute_name, self._groups_previous_values.get(group, None))\
-                for group in self.report.groups if group in self._groups_stack])
+        filter_dict = dict([(group.attribute_name, value) for group, value in self._groups_working_values.items()])
 
         def filter_object(obj):
-            for k,v in filter.items():
+            for k,v in filter_dict.items():
                 if get_attr_value(obj, k) != v:
                     return False
 
             return obj
 
-        return [obj for obj in self.report.queryset if filter_object(obj)]
+        return filter(filter_object, self.report.queryset)
 
     # SubReports
 
@@ -725,3 +872,68 @@ class ReportGenerator(object):
     def keep_in_frame(self, widget, width, height, paragraphs, mode):
         raise Exception('Not implemented')
 
+    def fetch_from_cache(self):
+        if self.report.cache_status == CACHE_BY_QUERYSET:
+            hash_key = self.get_hash_key(self.report.queryset)
+        elif self.report.cache_status == CACHE_BY_RENDER:
+            hash_key = self.get_hash_key(self._rendered_pages)
+
+        cache = self.get_cache_backend()
+        buffer = cache.get(hash_key)
+
+        if buffer:
+            # Write to file stream
+            if hasattr(self.filename, 'write') and callable(self.filename.write):
+                self.filename.write(buffer)
+                return True
+                
+            # Write to file path
+            elif isinstance(self.filename, basestring):
+                fp = file(self.filename, 'w')
+                fp.write(buffer)
+                fp.close()
+                return True
+
+    def cached_before_render(self):
+        """Check and loads the generated report from caching system before call method
+        'render_bands'"""
+
+        if not self.cache_enabled or self.report.cache_status != CACHE_BY_QUERYSET:
+            return False
+
+        return self.fetch_from_cache()
+
+    def cached_before_generate(self):
+        """Check and loads the generated report from caching system before call method
+        'generate_pages'"""
+
+        if not self.cache_enabled or self.report.cache_status != CACHE_BY_RENDER:
+            return False
+
+        return self.fetch_from_cache()
+
+    def store_in_cache(self, content):
+        """Sends the canvas content to write in the cache backend"""
+
+        if not self.cache_enabled or self.report.cache_status == CACHE_DISABLED:
+            return
+
+        if self.report.cache_status == CACHE_BY_QUERYSET:
+            hash_key = self.get_hash_key(self.report.queryset)
+        elif self.report.cache_status == CACHE_BY_RENDER:
+            hash_key = self.get_hash_key(self._rendered_pages)
+
+        cache = self.get_cache_backend()
+
+        return cache.set(hash_key, content)
+
+    def get_hash_key(self, objects):
+        """Calculates the hash_key, appending/prepending something if necessary"""
+        return make_hash_key(self.report, objects)
+
+    def get_cache_backend(self):
+        return get_cache_backend(
+                self.report.cache_backend,
+                cache_file_root=self.report.cache_file_root,
+                )
+
